\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{control}[2022/5/22 Examens i controls]

\LoadClass[parskip=half,twoside=semi,textsize=10pt]{scrartcl}
\RequirePackage{kvoptions}

%% Opcions de la classe

% afegeix capa compatibilitat amb antiga versió
\DeclareBoolOption[false]{compat}

% no es vol capçalera (yes/no)
\DeclareBoolOption[true]{namebox}

% llengua (catalan/english)
\DeclareBoolOption[true]{catalan}
\DeclareComplementaryOption{english}{catalan}

% espaiat (dot, square, dsquare)
\DeclareStringOption[dsquare]{grid}

% separacio grid (size)
\DeclareStringOption[7mm]{gridstep}

% to de la grid (value 0-100)
\DeclareStringOption[25]{gridcolor}

% format general (solutions, space, none)
\DeclareStringOption[space]{answers}

\ProcessKeyvalOptions*


%% Processa la llengua i precarrega opcions
\ifcontrol@catalan
  \PassOptionsToPackage{catalan,shorthands=off}{babel}
\else
  \PassOptionsToPackage{english}{babel}
\fi

%% Packages precarregats
\RequirePackage[T1]{fontenc}
\RequirePackage{lmodern}
\RequirePackage{babel}

\RequirePackage{tikz}
\RequirePackage{ifthen}
\RequirePackage{ccbyncsa}
\RequirePackage{xcolor}
\RequirePackage[logo=epsem,second=none]{fullupc}
\RequirePackage[paper=a4paper, centering, margin=14mm]{geometry}

%% Processa el format
\newif\ifcontrol@space
\newif\ifcontrol@sols
\ifthenelse{\equal{\control@answers}{solutions}}{
  \control@spacefalse\control@solstrue
}{
  \ifthenelse{\equal{\control@answers}{space}}{
    \control@spacetrue\control@solsfalse
  }{
    \ifthenelse{\equal{\control@answers}{none}}{
      \control@spacefalse\control@solsfalse
    }{
      \ClassError{control}{Wrong 'answers' option value}{Use:
        solutions, space, none}
    }
  }
}



%% Localize strings
\ifcontrol@catalan
  \newcommand{\@puntname}{punt}
  \newcommand{\@puntsname}{punts}
  \newcommand{\@exername}{Exercici}
  \newcommand{\@nomname}{NOM}
  \newcommand{\@cognomsname}{COGNOMS}
  \newcommand{\@grupname}{SUBGRUP}
  \newcommand{\@solname}{Solució}
\else
  \newcommand{\@puntname}{point}
  \newcommand{\@puntsname}{points}
  \newcommand{\@exername}{Question}
  \newcommand{\@nomname}{GIVEN NAME}
  \newcommand{\@cognomsname}{FAMILY NAME}
  \newcommand{\@grupname}{ENROLLMENT GROUP}
  \newcommand{\@solname}{Solution}
\fi

%% Enunciats de problemes
\newcounter{@exercici}
\renewcommand{\the@exercici}{\arabic{@exercici}}

\newenvironment{exercise}[1][0]{%
  \topsep=0pt \partopsep=0pt
  \begin{trivlist}\item%
    \refstepcounter{@exercici}%
    \textbf{\@exername~\the@exercici%
      \ifthenelse{\equal{#1}{0}}{}%
      {~[#1 \ifthenelse{\equal{#1}{1}}{\@puntname}{\@puntsname}]}.}%
    \quad\ignorespaces%
}{%
\end{trivlist}
}

%% Solucions
\newenvironment{answer}{
  \ifcontrol@sols
  \begin{trivlist}\item
    \textsc{\@solname:}\par
    \footnotesize
  \fi
}{
  \ifcontrol@sols
  \end{trivlist}
  \fi
}

%% Llista de respostes tancades
\newenvironment{choice}{%
  \penalty-400
  \renewcommand{\labelitemi}{\raisebox{0.3ex}{$\bigcirc$}}%
  \begin{itemize}
}{%
  \end{itemize}
}

%% Caselles de formulari
\newcommand{\@caselles}[1]{%
  \tikz[very thick,black!20,scale=.6]
  \draw[xscale=0.6] (0,0) grid +(#1,1);
}
\newcommand{\@camp}[2]{%
  \begin{tabular}{l}\scriptsize #1:\\\@caselles{#2}\end{tabular}}


%% Espais d'escriptura

\newcommand{\@quadricula}[1]{%
  \pgfmathsetmacro{\hnumber}{div(\linewidth,\control@gridstep)}%
  % escalat amb un pèl de defecte per compensar petits errors
  % de càlcul que produeixen overfull boxes
  \pgfmathsetmacro{\escalat}{\control@gridstep/10.02mm}%
  \ifthenelse{\equal{\control@grid}{dot}}{%
    \begin{tikzpicture}[scale=\escalat]
      \foreach \x in {0,...,\hnumber}
      \foreach \y in {0,...,#1}
      {
        \fill[gray!\control@gridcolor] (\x,\y) circle (0.6mm);
      }
    \end{tikzpicture}%
  }{%
    \ifthenelse{\equal{\control@grid}{square}}{%
      \begin{tikzpicture}%
        \draw[scale=\escalat, line width=0.3mm, gray!\control@gridcolor]
        (0,0) grid (\hnumber,#1);
      \end{tikzpicture}%
    }{%
      \ifthenelse{\equal{\control@grid}{dsquare}}{%
        \begin{tikzpicture}%
          \draw[scale=\escalat, line width=0.5mm,
          gray!\control@gridcolor]
          (0,0) grid (\hnumber,#1);
          \draw[scale=\escalat, line width=0.2mm,
          gray!\control@gridcolor, step=0.5cm]
          (0,0) grid (\hnumber,#1);
        \end{tikzpicture}%
      }{%
        \ClassError{control}{Wrong grid option value}{%
        Allowed values are: dot, square, dsquare}
      }%
    }%
  }%
}

% Calcula l'espai que resta a pàgina durant shipout
\newdimen\@remainingheight
\newcommand*{\@calcremainingheight}{%
  \ifdim\pagegoal=\maxdimen
    \@remainingheight\dimexpr\textheight-\control@gridstep\relax
  \else
  % edit 2: replaced -\baselineskip by -\lineskip-0.4pt
  % edit 3: removed -\parskip
     \@remainingheight\dimexpr\pagegoal-\pagetotal-\lineskip-0.4pt\relax
      %\@remainingheight\dimexpr\textheight-\pagetotal-\lineskip-0.4pt\relax
  \fi
}

% Línies en blanc amb quadricula
\newcommand{\answerlines}[1]{%
  \ifcontrol@space\par\medskip\hfill\@quadricula{#1}\hfill\break
  \vspace*{-0.5\control@gridstep}\fi%
}

% Quadricula fins final de pàgina
\newcommand{\answerlinestoeop}{
  \ifcontrol@space\par\medskip%
  \@calcremainingheight%
  \pgfmathsetmacro{\@alcada}{div(\@remainingheight,\control@gridstep)-1}%
  \hfill\@quadricula{\@alcada}\hfill\break\fi%
}

% Pagina sencera amb quadricula
\newcommand{\answerlinespage}{
  \ifcontrol@space
  \pgfmathsetmacro{\@alcada}{div(\textheight,\control@gridstep)}%
  \noindent\parbox[t]{\textwidth}%
  {\hfill\@quadricula{\@alcada}\hfill\break}\fi
}

%% Capçalera de la pràctica
\newcommand{\degree}[1]{\def\@degree{#1}}
\newcommand{\examtime}[1]{\def\@duration{#1}}

\renewcommand{\maketitle}{
  \vspace*{5mm}
  \begin{minipage}{1.0\linewidth}
    \begin{tabular*}{\linewidth}{l@{\extracolsep{\fill}}c@{\extracolsep{\fill}}r}
      & \textbf{\Large\@title} &             \\
      \makebox[14em][l]{\@date} & \@degree & \makebox[14em][r]{\textsc{\@duration}}\\
    \end{tabular*}
    \ifcontrol@namebox
    \rule{\linewidth}{1pt}
    \@camp{\@cognomsname}{36}\par
    \@camp{\@nomname}{20} \hfil \@camp{\@grupname}{2}\par
    \smallskip
    \fi
    \rule{\linewidth}{1pt}
  \end{minipage}
}


%% Setup final

\pagestyle{empty}

%\metadades{\@title}{http://ocw.itic.cat}

\raggedbottom{}

\renewcommand{\labelenumi}{\alph{enumi})}
\renewcommand{\labelenumii}{\roman{enumii})}
\renewcommand{\labelenumiii}{\arabic{enumiii})}

\AtBeginDocument{%
  %  nasty trick to avoid:
  %  https://www.mail-archive.com/tex-hyphen@tug.org/msg01801.html
  \ifcontrol@catalan\lccode`'=0\fi
  \sloppy
  \maketitle{}
}


%% Capa de compatibilitat amb les versions anteriors

\ifcontrol@compat
\newenvironment{problema}{\exercise}{\endexercise}
\newenvironment{solucio}{\answer}{\endanswer}
\newenvironment{respostes}{\choice}{\endchoice}
\NewCommandCopy{\quadricula}{\answerlines}
\NewCommandCopy{\quadrifdp}{\answerlinestoeop}
\NewCommandCopy{\quadritot}{\answerlinespage}
\NewCommandCopy{\estudis}{\degree}
\NewCommandCopy{\durada}{\examtime}
\fi
